from datetime import datetime
from typing import List, Annotated

from fastapi_users.db import SQLAlchemyBaseUserTable
from sqlalchemy import ForeignKey, text
from sqlalchemy.orm import Mapped, mapped_column, relationship, DeclarativeBase


intpk = Annotated[int, mapped_column(primary_key=True)]
created_at = Annotated[
    datetime,
    mapped_column(server_default=text("TIMEZONE('utc', now())")),
]
updated_at = Annotated[
    datetime,
    mapped_column(
        server_default=text("TIMEZONE('utc', now())"),
        onupdate=datetime.utcnow,
    ),
]


class Base(DeclarativeBase):
    pass


class UserRoleAssociation(Base):
    """
    Модель таблицы связи Many2Many для Role & User
    """

    __tablename__ = "user_role_association"
    user_id: Mapped[int] = mapped_column(
        ForeignKey("user.id"),
        primary_key=True,
    )
    role_id: Mapped[int] = mapped_column(
        ForeignKey("role.id"),
        primary_key=True,
    )
    role: Mapped["Role"] = relationship()


class Role(Base):
    """
    Модель таблицы role
    """

    __tablename__ = "role"

    id: Mapped[intpk]
    role: Mapped[str]


class User(SQLAlchemyBaseUserTable[int], Base):
    """
    Модель таблицы user
    """

    __tablename__ = "user"
    id: Mapped[intpk]
    roles: Mapped[List["UserRoleAssociation"]] = relationship()


class ClientLocationAssociation(Base):
    __tablename__ = "client_location_association"
    client_id: Mapped[int] = mapped_column(
        ForeignKey("client.id"),
        primary_key=True,
    )
    location_id: Mapped[int] = mapped_column(
        ForeignKey("location.id"), primary_key=True
    )
    location: Mapped["Location"] = relationship()


class Client(Base):
    """
    Клиенты и отделы организации у которых обслуживается оборудование
    """

    __tablename__ = "client"
    id: Mapped[intpk]
    name: Mapped[str]
    equipment: Mapped[int]
    locations: Mapped[List[ClientLocationAssociation]] = relationship()


class Location(Base):
    """
    Адреса клиентов где установленно оборудование
    """

    __tablename__ = "location"
    id: Mapped[intpk]
    address: Mapped[str]
    client: Mapped[int]


class Equipment(Base):
    __tablename__ = "equipment"
    id: Mapped[intpk]
    name: Mapped[str]
    eq_type: Mapped[int]
    # location: Mapped[List["Location"]] = relationship()
    put_into_operation_at: Mapped[datetime]
    # user_id: Mapped[int] = mapped_column(ForeignKey("user.id"))
    # child: Mapped["User"] = relationship()
